
#!/usr/bin/env python

import string, math

class DipVariant:
    name_table = {}
    id = 0
    
    def __init__(self, name, first_year, totalcentres, start_sc, homepage = 0):
        DipVariant.name_table[string.upper(name)] = self

        self.id = DipVariant.id
        DipVariant.id = DipVariant.id + 1
        self.name = name
        self.start_sc = start_sc
        self.first_year = first_year
        self.totalcentres = totalcentres
        self.homepage = homepage
        if start_sc != {}:
            self.is_drop = 0
        else:
            self.is_drop = 1

    def html_ref(self):
        if self.is_drop:
            return self.name
        else:
            return html_link(self.name, 'variant' + str(self.id) + '.html')

    def html_ref_hist(self):
        if self.is_drop:
            return self.name
        else:
            return html_link(self.name, '../hist/variant' + str(self.id)
	    + '.html')


    def find_power(self, str):
        str = string.upper(str)
        if self.start_sc.has_key(str):
            return str

        if self.is_drop:
            self.start_sc[str] = 1
            return str
        
	str = str[:1]
        if self.start_sc.has_key(str):
            return str

        panic()

    def average_score(self, game_list, pn):
    	total = 0
	for g in game_list:
	    total = total + g.pow_by_letter[pn].score
	return (total / len(game_list))

    def print_summary(self, game_list, f):
        if self.is_drop:
            panic()

        # Pick out the games that we're interested in.
        games = []
        for g in game_list:
            if g.variant == self:
                games.append(g)

        if len(games) == 0:
            return

	year = games[0].fileyear

        f.write(html_header(self.name + ' (' + year + ')'))
        f.write('<P ALIGN=CENTER>\n')
        f.write('<TABLE BORDER=1 WIDTH="50%">\n')
        f.write('<TR><TH COLSPAN=' + str(2 + len(games)) + '>' + self.name)
        if self.homepage:
            f.write(' <A HREF="' + self.homepage + '">(homepage)</A>')
        f.write('</TH></TR>\n')
        f.write('<TR><TH> Pwr. </TH>')
        for g in games:
            g.score()
            f.write('<TH>' + g.link_game() + '</TH>')

        f.write('<TH> Avg. </TH></TR>\n')

        # Compute average scores.
        power_scores = []
        for pn in self.start_sc.keys():
            power_scores.append((self.average_score(games, pn), pn))

        power_scores.sort()
        power_scores.reverse()

        for avg_score, pn in power_scores:
            f.write('<TR ALIGN=CENTER>')
            f.write('<TH>' + pn + '</TH>')
            for g in games:
                f.write(html_table_cell(zerodps(g.pow_by_letter[pn].score)))
            #   f.write(html_table_cell(twodps(g.pow_by_letter[pn].score)))

            f.write(html_table_cell('<EM>' + twodps(avg_score) + '</EM>'))
            f.write('</TR>\n')

        f.write(html_table_close())
	f.write('<p>' + html_link(season(year) + ' League table', 's' + year + '.html'))
	f.write('<br>' + html_link('Historical entry',
	    '../hist/variant' + str(self.id) + '.html')) 
        f.write(html_footer())

    
    def print_hist_summary(self, game_list, f):
        if self.is_drop:
            panic()

        # Pick out the games that we're interested in.
        games = []
        for g in game_list:
            if g.variant == self:
                games.append(g)

        if len(games) == 0:
            return

        f.write(html_header(self.name))
        f.write('<P ALIGN=CENTER>\n')
        f.write('<TABLE BORDER=1 WIDTH="50%">\n')
        f.write('<TR><TH COLSPAN=' + str(2 + len(games)) + '>' + self.name)
        if self.homepage:
            f.write(' <A HREF="' + self.homepage + '">(homepage)</A>')
        f.write('</TH></TR>\n')
        f.write('<TR><TH> Pwr. </TH>')
        for g in games:
            g.score()
            f.write('<TH>' + g.link_game_hist() + '</TH>')

        f.write('<TH> Avg. </TH></TR>\n')

        # Compute average scores.
        power_scores = []
        for pn in self.start_sc.keys():
            power_scores.append((self.average_score(games, pn), pn))

        power_scores.sort()
        power_scores.reverse()

        for avg_score, pn in power_scores:
            f.write('<TR ALIGN=CENTER>')
            f.write('<TH>' + pn + '</TH>')
            for g in games:
                f.write(html_table_cell(zerodps(g.pow_by_letter[pn].score)))
            #   f.write(html_table_cell(twodps(g.pow_by_letter[pn].score)))

            f.write(html_table_cell('<EM>' + twodps(avg_score) + '</EM>'))
            f.write('</TR>\n')

        f.write(html_table_close())
	f.write('<p>' + html_link('Front', 'front.html'))
        f.write(html_footer())


def find_variant(str):
    if string.upper(str[:4]) == 'DROP':
        str = str[5:]
        par = DipVariant.name_table[string.upper(str)]
        return DipVariant('Drop ' + str, par.first_year, par.totalcentres,
                          {}, 0)
    
    return DipVariant.name_table[string.upper(str)]

DipVariant('Standard', 1901, 34,
           {'A': 3, 'E': 3, 'F': 3, 'G': 3, 'I': 3, 'R': 4, 'T': 3},
           'http://www.diplomacy-archive.com/resources/strategy.htm')
DipVariant('Bretwalda', 621, 50,
           {'D': 4, 'E': 3, 'G': 3, 'K': 3, 'L': 3, 'M': 4, 'N': 3,
	   'P': 3, 'W': 3},
           'http://web.libertysurf.se/baches/bretwald.htm')
DipVariant('Modern', 1995, 64,
           {'B': 4, 'E': 3, 'F': 4, 'G': 4, 'I': 4, 'P': 3, 'R': 5,
            'S': 3, 'T': 4, 'U': 4},
           'http://www.modernhof.webprovider.com/')
DipVariant('Devolution', 2001, 51,
           {'A': 3, 'C': 3, 'I': 3, 'L': 3, 'M': 5, 'S': 4, 'W': 4, 'Y': 4})
DipVariant('Cantabrigia', 2001, 32,
           {'E': 3, 'T': 3, 'R': 3, 'S': 4, 'K': 3, 'P': 3},
	   'http://www.pilgrim.demon.co.uk/alex/cantab.htm')
DipVariant('Classical', 1, 35,
           {'C': 3, 'E': 3, 'M': 3, 'R': 3, 'S': 3})
DipVariant('Octarine', 1, 42,
           {'A': 3, 'C': 3, 'G': 3, 'H': 3, 'K': 3, 'M': 4, 'O': 3, 'X': 3},
           'http://www.ftech.net/~monark/dip/octarine/')
DipVariant('Octarine V4', 1, 40,
           {'A': 3, 'C': 3, 'G': 3, 'H': 3, 'M': 4, 'N': 3, 'O': 3, 'X': 3},
           'http://www.ftech.net/~monark/dip/octarine/')
DipVariant('Ancient Mediterranean', 1, 35,
           {'R': 3, 'C': 3, 'G': 3, 'P': 3, 'E': 3},
           'http://www.variantbank.com/rules/a/ancient_mediterranean.htm')
DipVariant('Double Modern', 1, 64,
	   {'BU': 8, 'EG': 7, 'FT': 8, 'IR': 9, 'PS': 6})
DipVariant('Sail Ho!', 1, 16,
	   {'E': 2, 'N': 2, 'S': 2, 'W': 2},
	   'http://www.jump.to/SailHo')
DipVariant('Aberration', 1901, 52,
	   {'B': 3, 'C': 3, 'E': 3, 'H': 3, 'I': 3, 'P': 3, 'S': 3, 'U': 3, 
	   'Z':3},
	   'http://devel.diplom.org/Email/judge/info.aberration.html')
DipVariant('Colonial', 1870, 58,
	   {'B': 6, 'C': 5, 'F': 3, 'H': 3, 'J': 4, 'R': 5, 'T': 3},
	   'http://devel.diplom.org/Email/judge/info.colonial.html')
DipVariant('Heptarchy', 651, 37,
	   {'A': 3, 'C': 3, 'I': 3, 'M': 4, 'N': 3, 'S': 3, 'W': 3},
	   'http://web.libertysurf.se/baches/heptarch.htm')
DipVariant('Youngstown', 1901, 72,
	   {'A': 4, 'C': 3, 'E': 4, 'F': 4, 'G': 4, 'I': 4, 'J': 3,
	   'N': 3, 'R': 6, 'T': 4},
	   'http://devel.diplom.org/Email/judge/info.youngstown.html')
DipVariant('Lunacy V1', 1, 28,
	{'C': 3, 'G': 3, 'H': 3, 'N': 3, 'P': 3, 'T': 3})

def isapint(str):
    import re
    return re.search(r'[^0-9]', str) == None

def twodps(i):
    import re
    s = str(round(i, 2))
    while re.search(r'\d+.\d\d', s) == None:
        s = s + '0'
    return s

def zerodps(i):
    return str(round(i, 0))[:-2]

def cook_line(s):
    # Remove comments.
    try:
        s = s[:string.index(s, '#')]
    except ValueError:
        pass

    # Split into words.
    s = string.split(s)

    return s

def cmpbyscore(a, b):
    return -cmp(a.score, b.score)

def cmpbyprescore(a, b):
    return -cmp(a.prescore, b.prescore)

def cmpbyname(a, b):
    return cmp(a.name, b.name)

def cmpbysc(a, b):
    return -cmp(a.end_sc, b.end_sc)


# HTML output related functions.

def html_header(title):
    return ('<!DOCTYPE HTML PUBLIC ' +
            '"-//W3C//DTD HTML 4.01 Transitional//EN" ' +
            '"http://www.w3.org/TR/html4/loose.dtd">\n' +
            '<HTML>\n' +
            '<HEAD>\n' +
            '<META http-equiv="Content-Type" ' +
            'content="text/html; charset=ISO-8859-1">\n' + 
            '<TITLE> ' + title + ' </TITLE>\n' +
            '</HEAD>\n' +
            '<BODY BACKGROUND="white_pa.gif">\n')

def html_footer():
    return ('<p><a href="http://validator.w3.org/check/referer">' +
            '<img border=0 src="http://www.w3.org/Icons/valid-html401"' +
            ' alt="Valid HTML 4.01!" height=31 width=88>' +
            '</a> <a href="../index.html">Back to the CU DipSoc home page.</a>' +
            '</p>\n</BODY>\n</HTML>\n')

def html_link(caption, url):
    return '<A HREF="' + url + '">' + caption + '</A>'

def html_table_open():
    pass

def html_table_close():
    return '</TABLE>\n'

def html_table_cell(contents):
    return '<TD>' + contents + '</TD>'

def html_table_cell_open():
    return '<TD>'

def html_table_cell_close():
    return '</TD>'

def html_table_header(contents):
    return '<TH>' + contents + '</TH>'

def season(year):
    return str(int(year)-1) + '/' + year


class DipPower:
    def __init__(self, game, powername, player, start_sc, end_sc, last_year):
        self.game = game
        self.name = powername
        self.player = player
        self.start_sc = start_sc
        self.end_sc = end_sc
        self.last_year = last_year

    def print_entry(self, f):
        
        f.write ('<TR ALIGN="CENTER"><TD>' + self.name +
                 '</TD><TD ALIGN="LEFT">' + self.player.html_ref() +
                 '</TD><TD>' + str(self.end_sc) + '</TD>')

        if self.game.prescored:
            f.write ('<TD>' + twodps(self.prescore) + '</TD>')
        if self.game.scored:
            f.write ('<TD>' + twodps(self.score) + '</TD>')

        f.write ('<TD>' + str(self.last_year) + '</TD></TR>\n')

class DipGame:
    def __init__(self, filename, fileyear):
        print '%%% Loading game: s' + fileyear + '/' + filename, '%%%'
        
        self.scored = 0
        self.prescored = 0
	self.nocount = 0
        self.pow_by_letter = {}
        self.pow_by_player = {}
        self.name = filename
	self.fileyear = fileyear

        f = open('s'+ fileyear + '/' + filename, 'r')
        rawlines = f.readlines()
        f.close()

        # Cook the input.
        lines = []
        for s in rawlines:
            s = cook_line(s)
            
            # Don't bother with blank lines.
            if s != []:
                lines.append(s)

        # The first line contains the variant and the last year played.

	if (lines[0][-1] == '*'):
	    self.nocount = 1
	    del lines[0][-1]

        if not (isapint(lines[0][-1]) or (lines[0][-1][0] == '+' and isapint(lines[0][-1][1:]))):
            print lines[0][-1][0]
            print lines[0][-1][1:]
            print '*** The first non-comment line should contain the variant and the first year played. ***'
            panic()

	# Extract the variant name.
        self.variant = find_variant(string.join(lines[0][0:-1]))

        # Extract the last year played.
        if lines[0][-1][0] == '+':
            self.last_year = int(lines[0][-1][1:]) + self.variant.first_year - 1
        else:
            self.last_year = int(lines[0][-1])
	
	if(self.last_year < self.variant.first_year):
	    print '*** Game ends before it begins ***'
	    panic()
	
	# Get rid of the first line altogether.
        del lines[0]

        for s in lines:
            powername = self.variant.find_power(s[0])
            del s[0]
            
            if self.pow_by_letter.has_key(powername):
                print '*** Duplicate power:', powername, '***'
                panic()

            start_sc = self.variant.start_sc[powername]

            i = 0
            while i < len(s) and not isapint(s[i]):
                i = i + 1

            if i == len(s):
                panic()

            playername = string.join(s[0 : i])
            if playername == '':
                print '*** Power', powername, 'has no name! ***'
                panic()

	    end_sc = int(s[i])
            if end_sc == 0:
	    	if s[i+1][0] == '+':
                    plast_year = self.variant.first_year + int(s[i + 1]) - 1
		else:    
                    plast_year = int(s[i + 1])
            else:
                plast_year = self.last_year

            player = find_player(playername, self)
            power = DipPower(self, powername, player,
                             start_sc, end_sc, plast_year)

            self.pow_by_letter[powername] = power
            self.pow_by_player[player] = power
            
        # Sanity checks.
        d = len (self.pow_by_letter) - len(self.variant.start_sc)

        if d > 0:
            print '*** Too many powers in file! ***'
            panic()
        elif d < 0:
            print '*** Too few powers in file! ***'
            print result
            panic()

        # Count the number of centres held at game end.
        n = 0
        for r in self.pow_by_letter.values():
            n = n + r.end_sc

        if n > self.variant.totalcentres:
            print '*** Total number of occupied centres at game end exceeds total number of available centres! ***'
            # panic()
        elif n < self.variant.totalcentres:
            print '%%% Note: not all centres were occupied at game end. %%%'

    def power_by_playername(self, playername):
        for p in self.pow_by_letter.values():
            if playername == p.player:
                return p
        panic()

    def prescore(self):
        if self.prescored:
            return self

        winning_end_sc = 0
        winning_start_sc = self.variant.totalcentres

        for p in self.pow_by_letter.values():
            if p.end_sc > winning_end_sc:
                winning_end_sc = p.end_sc
                winning_start_sc = p.start_sc
            elif p.end_sc == winning_end_sc:
                if p.start_sc < winning_start_sc:
                    winning_start_sc = p.start_sc

        for p in self.pow_by_letter.values():
            pscore = ((0.2 * (p.last_year - self.variant.first_year + 1))
                      / (self.last_year - self.variant.first_year + 1))

            mul1 = ((0.7 * p.end_sc) / winning_end_sc)

            mul2 = (1 + (p.end_sc - p.start_sc)
                    / (7.0 * (winning_end_sc - winning_start_sc)))

            pscore = pscore + mul1 * mul2

            p.prescore = pscore

        self.prescored = 1
        return self

    def score(self):
        if self.scored:
            return self

        self.prescore()

        total = 0
        for p in self.pow_by_letter.values():
            total = total + p.prescore

        factor = len(self.pow_by_letter) * 50.0 / total

        for p in self.pow_by_letter.values():
            p.score = factor * p.prescore

        self.scored = 1
        return self

    def print_summary(self, f):
        fields = 4

        if self.prescored:
            fields = fields + 1

        if self.scored:
            fields = fields + 1

        f.write (html_header(self.name + ' (' + self.fileyear +')'))
        f.write ('<P ALIGN=CENTER>\n')
        f.write ('<TABLE BORDER="1" WIDTH="50%">\n')
        f.write ('<TR ALIGN=CENTER><TH COLSPAN=' + str(fields) + '>' + self.name + '\n')
        f.write ('<TR><TH COLSPAN=' + str(fields) + '>')
        f.write (self.variant.html_ref())
        f.write (', last year played: ' + str(self.last_year) + '\n')

        if self.scored:
            cf = cmpbyscore
        elif self.prescored:
            cf = cmpbyprescore
        else:
            cf = cmpbysc

        l = self.pow_by_letter.values()[:]
        l.sort(cf)

        for p in l:
            p.print_entry(f)

        f.write (html_table_close())
	f.write ('<p>' + html_link(season(self.fileyear) + ' League table',
	    's' + self.fileyear + '.html'))
        f.write (html_footer())


    def link_game(self):
    	str = html_link(self.name, self.name + '.html')
	if self.nocount:
	    str = str + '*'
	return str
	
    def link_game_hist(self):
    	str = html_link(self.name, '../s' + self.fileyear + '/' + 
	self.name + '.html')
	if self.nocount:
	    str = str + '*'
	return str


class DipPlayer:
    master_list = {}
    id = 0
    def __init__(self, name):
        print '%%% New player:', name, '%%%'
        if DipPlayer.master_list.has_key(name):
            panic()
        DipPlayer.master_list[name] = self
        self.name = name
	first_name = string.split(name)[0]
	if names.has_key(first_name):
	    self.gender = names[first_name]
	else:    
	    names[first_name] = 'u'
	    self.gender = 'u'
        self.games = {}
	self.hist_games = []
        self.id = DipPlayer.id
        DipPlayer.id = DipPlayer.id + 1

    def print_summary(self, year, f):
        f.write(html_header(self.name + ' (' + year + ')'))

        f.write('<P ALIGN="CENTER">\n')
        f.write('<TABLE BORDER=1 WIDTH="50%">\n')
        f.write('<TR ALIGN=CENTER><TH COLSPAN=5>' + self.name + '</TH></TR>\n')

        for g in self.games[year]:
            g.score()
            f.write('  <TR ALIGN=CENTER>' +
                    '<TD>' + g.link_game() + '</TD>' +
                    '<TD ALIGN=LEFT>' +
                    g.variant.html_ref() +
                    '</TD>\n' +
                    '    <TD> ' + g.pow_by_player[self].name + ' </TD>' +
                    '<TD> ' + twodps(g.pow_by_player[self].prescore) + ' </TD>' +
                    '<TD> ' + twodps(g.pow_by_player[self].score) + ' </TD>' +
                    '</TR>\n')

        f.write('<TR><TH COLSPAN=4>Average:</TH><TD ALIGN=CENTER><EM>' +
                twodps(self.score) + '</EM></TD></TR>\n')

        f.write(html_table_close())
	f.write('<p>' + html_link(season(year) + ' League table', 's' + year + '.html'))
	f.write('<br>' + html_link('Historical entry', 
	    '../hist/player' + str(self.id) + '.html'))
        f.write(html_footer())
    
    
    def print_hist_summary(self, f):
        f.write(html_header(self.name))

        f.write('<P ALIGN="CENTER">\n')
        f.write('<TABLE BORDER=1 WIDTH="50%">\n')
        f.write('<TR ALIGN=CENTER><TH COLSPAN=5>' + self.name + '</TH></TR>\n')

        for g in self.hist_games:
            g.score()
            f.write('  <TR ALIGN=CENTER>' +
                    '<TD>' + g.link_game_hist() + '</TD>' +
                    '<TD ALIGN=LEFT>' +
                    g.variant.html_ref() +
                    '</TD>\n' +
                    '    <TD> ' + g.pow_by_player[self].name + ' </TD>' +
                    '<TD> ' + twodps(g.pow_by_player[self].prescore) +
		    ' </TD>' + '<TD> ' + twodps(g.pow_by_player[self].score)
		    + ' </TD>' + '</TR>\n')

        f.write('<TR><TH COLSPAN=4>Average:</TH><TD ALIGN=CENTER><EM>' +
                twodps(self.score) + '</EM></TD></TR>\n')

        f.write(html_table_close())
	f.write('<p>' + html_link('Front', 'front.html'))
        f.write(html_footer())
    
    def add_game(self, game):
    	if not (self.games.has_key(game.fileyear)):
	    self.games[game.fileyear] = []
    	self.games[game.fileyear].append(game)
	self.hist_games.append(game)

    def html_ref(self):
    	return html_link(self.name, 'player' + str(self.id) + '.html')

def find_player(name, game):
    if DipPlayer.master_list.has_key(name):
        p = DipPlayer.master_list[name]
    else:
        p = DipPlayer(name)
    p.add_game(game)
    return p


def make_league_table(glist, year, f):
    # Score each game.
    for g in glist:
        g.score()

    plist = []
    for p in DipPlayer.master_list.values():
    	if p.games.has_key(year):
	    plist.append(p)

    # For each player, compute the average score and create a summary.
    for player in plist:
        total = 0
	counted_games = 0
        for g in player.games[year]:
	    if not (g.nocount):
            	counted_games = counted_games + 1
		total = total + g.pow_by_player[player].score
	
	# If someone has played only unscored games use those instead
	if counted_games == 0:
	    for g in player.games[year]:
	    	counted_games = counted_games + 1
		total = total + g.pow_by_player[player].score
	
	player.score = total / counted_games
       	pf = open('s' + year + '/player' + str(player.id) + '.html','w')
        player.print_summary(year, pf)
        pf.close()

    # Generate a summary for each non-drop variant.
    for v in DipVariant.name_table.values():
        if v.is_drop:
            continue
        vf = open('s' + year + '/variant' + str(v.id) + '.html','w')
        v.print_summary(glist, vf)
        vf.close()

    if not year:
    	return

    # Sort the list of players by score.    
    plist.sort(cmpbyscore)
    
    f.write(html_header('League Table'))

    f.write('<P ALIGN=CENTER>\n')
    f.write('<TABLE BORDER=1>\n')

    # Print the headers of the table.
    f.write ('<TR><TH COLSPAN="' + str(4 + len(glist)) + '">League Table for ')
    f.write (season(y) + '</TH>\n')
    
    f.write ('<TR><TH>Pos.</TH><TH ALIGN="LEFT"> Name </TH><TH> Pld. </TH><TH> Avg. </TH>\n')
    for g in glist:
        f.write ('<TH>' + g.link_game() + '</TH>\n')	

    f.write('</TR>\n')

    # If we have >= 10 games then we need to make output a bit smaller
    # so we can make it fit without too much trouble in one screenful

    if len(glist) >= 13:
        roundfunc = zerodps
        emptycell = '  '
    elif len(glist) >= 10:
        roundfunc = lambda f: str(round(f, 1))
        emptycell = '    '
    else:
        roundfunc = twodps
        emptycell = '     '

    # For each player, in sorted order...
    table_pos = 1
    for player in plist:
    	if len(player.games[year]) <= 1:
		continue
        # Print name, number of games, and rounded score.
        f.write ('<TR ALIGN="CENTER"><TH>')        
        if len(player.games[year]) >= 3:
            # You must play at least 3 games to get a ranking.
            f.write(str(table_pos))
            table_pos = table_pos + 1
        f.write ('</TH><TD ALIGN="LEFT">' + player.html_ref() + '</TD>\t')
        f.write ('<TD>' + str(len(player.games[year])) + '</TD>')
        f.write ('<TD><EM>' + twodps(player.score) + '</EM></TD>\t')

        # For each game played, check if this player played in it and
        # if so print the rounded score in that game.  Otherwise, just
        # leave the cell empty.
        for g in glist:
            f.write (html_table_cell_open())
            if g in player.games[year]:
                f.write (roundfunc(g.pow_by_player[player].score))
            else:
                f.write(emptycell)
            f.write (html_table_cell_close())
        f.write('</TR>\n')

    # Bottom of the table and footers.
    f.write (html_table_close()
             + 'Note: You must play at least 3 games to get a ranking.\n'
	     + 'Games marked * are not included in the average score.\n'
	     + '<p>' + html_link('Front', '../hist/front.html')
             + html_footer())


def make_hist_league_table(glist, f):
    # Score each game.
    for g in glist:
        g.score()
    plist = DipPlayer.master_list.values()

    # For each player, compute the average score and create a summary.
    for player in plist:
        total = 0
	counted_games = 0
        for g in player.hist_games:
	    if not (g.nocount):
            	counted_games = counted_games + 1
		total = total + g.pow_by_player[player].score
	
	# If someone has played only unscored games use those instead
	if counted_games == 0:
	    for g in player.hist_games:
	    	counted_games = counted_games + 1
		total = total + g.pow_by_player[player].score
	
	player.score = total / counted_games
       	pf = open('hist/player' + str(player.id) + '.html','w')
        player.print_hist_summary(pf)
        pf.close()

    # Generate a summary for each non-drop variant.
    for v in DipVariant.name_table.values():
        if v.is_drop:
            continue
        vf = open('hist/variant' + str(v.id) + '.html','w')
        v.print_hist_summary(glist, vf)
        vf.close()

    # Sort the list of players by score.    
    plist.sort(cmpbyscore)
    
    f.write(html_header('League Table'))

    f.write('<P ALIGN=CENTER>\n')
    f.write('<TABLE BORDER=1>\n')

    # Print the headers of the table.
    f.write ('<TR><TH COLSPAN="' + str(4 + len(glist)) + '">')
    f.write ('Historical League Table</TH>\n')
    
    f.write ('<TR><TH>Pos.</TH><TH ALIGN="LEFT"> Name </TH><TH> Pld. </TH><TH> Avg. </TH>\n')
    for g in glist:
        f.write ('<TH>' + g.link_game_hist() + '</TH>\n')	

    f.write('</TR>\n')

    # If we have >= 10 games then we need to make output a bit smaller
    # so we can make it fit without too much trouble in one screenful

    if len(glist) >= 13:
        roundfunc = zerodps
        emptycell = '  '
    elif len(glist) >= 10:
        roundfunc = lambda f: str(round(f, 1))
        emptycell = '    '
    else:
        roundfunc = twodps
        emptycell = '     '

    # For each player, in sorted order...
    table_pos = 1
    for player in plist:
        # Print name, number of games, and rounded score.
        f.write ('<TR ALIGN="CENTER"><TH>')
	if len(player.hist_games) < 3:
	    continue
        if len(player.hist_games) >= 7:
            # You must play at least 7 games to get a ranking.
            f.write(str(table_pos))
            table_pos = table_pos + 1
        f.write ('</TH><TD ALIGN="LEFT">' + player.html_ref() + '</TD>\t')
        f.write ('<TD>' + str(len(player.hist_games)) + '</TD>')
        f.write ('<TD><EM>' + twodps(player.score) + '</EM></TD>\t')

        # For each game played, check if this player played in it and
        # if so print the rounded score in that game.  Otherwise, just
        # leave the cell empty.
        for g in glist:
            f.write (html_table_cell_open())
            if g in player.hist_games:
                f.write (roundfunc(g.pow_by_player[player].score))
            else:
                f.write(emptycell)
            f.write (html_table_cell_close())
        f.write('</TR>\n')

    # Bottom of the table and footers.
    f.write (html_table_close()
             + 'Note: You must play at least 7 games to get a ranking.\n'
	     + 'Games marked * are not included in the average score.\n'
	     + '<p>' + html_link('Front', 'front.html')
             + html_footer())

def make_hist_front(glist, f):
    f.write(html_header('Statistics'))
    f.write('<p><h3>League tables</h3>')
    f.write('<p>These league tables are generated from the written records ' +
    	'that DipSoc has of games played from Michaelmas 1996 onwards. ' +
	'Please point out any errors (misspelled names, broken links, etc) to ' +
	'<a href="mailto:soc-dip-committee@lists.cam.ac.uk">the committee</a>.')
	
    f.write('<p>')	
    for y in years:
    	f.write(html_link('League table for ' + season(y) + ' season',
	'../s' + y + '/s' + y + '.html') + '<br>')

    f.write('<p>An explanation of the algorithm used to score the games ' +
    	'can be found on <a href="../house_rules.html">this page</a>.<br>')
	    
    f.write('<p><h3>Variants</h3><p>')
    var_count = [0] * DipVariant.id
    for g in glist:
    	var_count[g.variant.id] = var_count[g.variant.id] + 1
    for v in DipVariant.name_table.values():
    	if v.is_drop:
	    continue
    	f.write(v.html_ref() + ' (' + str(var_count[v.id]) + ')<br>')

    stdvar = find_variant('Standard')
    stdnames = {'A': 'Austria', 'E': 'England', 'F': 'France',
    	'G': 'Germany', 'I': 'Italy', 'R': 'Russia', 'T': 'Turkey'}
    stdgames = []
    total_years = 0
    for g in glist:
    	total_years = total_years + g.last_year - g.variant.first_year + 1
	if g.variant == stdvar:
	    stdgames.append(g)
    max_score = 0
    for pn in stdvar.start_sc.keys():
    	avg = stdvar.average_score(stdgames, pn)
    	if avg > max_score:
	    max_score = avg
	    max_power = pn
    min_score = 100
    for g in stdgames:
    	for pn in stdvar.start_sc.keys():
	    if g.pow_by_letter[pn].score < min_score:
	    	min_score = g.pow_by_letter[pn].score
		min_power = pn
		min_game = g
    max_games = 0
    geoffs = []
    entries_m = entries_f = entries_n = entries_u = 0
    for p in DipPlayer.master_list.values():
    	if len(p.hist_games) > max_games:
	    max_games = len(p.hist_games)
	    max_player = p
	if p.name[:5] == 'Geoff':
	    geoffs.append(p)
	if p.gender == 'm':
	    entries_m = entries_m + len(p.hist_games)
	if p.gender == 'f':
	    entries_f = entries_f + len(p.hist_games)
	if p.gender == 'n':
	    entries_n = entries_n + len(p.hist_games)
	if p.gender == 'u':
	    entries_u = entries_u + len(p.hist_games)
    m_part = 100.0 * entries_m / (entries_m + entries_f)
    if entries_u:
    	print "Unknown first names written to hist/names_u."
    
    f.write('<p><h3>Q&A</h3><p>')
    f.write('Q: How many games are included in the statistics?<br>')
    f.write('A: ' + str(len(glist)) + '<br><br>')
    f.write('Q: What is the total number of years played in these games?<br>')
    f.write('A: ' + str(total_years) + '<br><br>')
    f.write('Q: What power in ' + stdvar.html_ref() +
        ' has the highest average score?<br>')
    f.write('A: ' + stdnames[max_power] + ' (' + twodps(max_score) + ')<br><br>')
    f.write('Q: What is the lowest score ever achieved in a ' + stdvar.html_ref() +
    	' game?<br>')
    f.write('A: ' + twodps(min_score) + ' (' + stdnames[min_power] + ' in ' +
    	min_game.link_game_hist() + ' in ' + 
	html_link(season(min_game.fileyear) + ' season', '../s' + min_game.fileyear
	+ '/s' + min_game.fileyear + '.html') + ')<br><br>')
    f.write('Q: Who has played the largest number of games?<br>')
    f.write('A: ' + max_player.html_ref() + ' (' + str(max_games) + ')<br><br>')
    f.write('Q: How many Geoffs are there in the DipSoc records?<br>')
    f.write('A: ' + str(len(geoffs)) + ' (')
    for i in range(len(geoffs)):
    	f.write(geoffs[i].html_ref())
	if i < (len(geoffs)-2):
	    f.write(', ')
	if i == (len(geoffs)-2):
	    f.write(' and ')
    f.write(')<br><br>')
    f.write('Q: Are men ridiculously overrepresented in the records?<br>')
    f.write('A: ')
    if m_part > 0.95:
    	f.write('Yes. ')
    if m_part < 0.55:
    	f.write('No. ')
    f.write(str(round(m_part, 1)) + '% of the player entries are men.<br><br>')
    f.write('Q: Who made these useless pages anyway?<br>')
    f.write('A: The scripts used to generate the league tables were originally ' +
    	'written by ' + DipPlayer.master_list['Gaute Strokkenes'].html_ref() +
	' and have been modified by ' +
	DipPlayer.master_list['Johannes Nordstrm'].html_ref() + '.<br><br>')
	
	
def read_names():
    g = open('hist/names_m', 'r')
    for l in g.readlines():
	names[l[:-1]] = 'm'
    g.close()
    g = open('hist/names_f', 'r')
    for l in g.readlines():
	names[l[:-1]] = 'f'
    g.close()
    g = open('hist/names_n', 'r')
    for l in g.readlines():
	names[l[:-1]] = 'n'
    g.close()


# Do the work.
files = {'1997': ('M3', 'M4a', 'M4b', 'M5b', 'M6a', 'M6b', 'M7b', 'M8', 'L0a',
	'L0b', 'L1a', 'L2a', 'L4', 'L6', 'L7', 'L8'),
	'1998': ('M2b', 'M2c', 'M3', 'M4a', 'M4b', 'M5', 'M6a', 'M6b',
	'M7a', 'M7b', 'M7d', 'M8a', 'M8c', 'L1a', 'L1b', 'L2a', 'L2b',
	'L3a', 'L4a', 'L4b', 'L5a', 'L5b', 'L6', 'L7', 'L8', 'L9b',
	'E1', 'E3', 'E4', 'E6', 'E7'),
	'1999': ('M2a', 'M2b', 'M3', 'M4', 'M5', 'M7', 'M8', 'M9', 'M10', 'L1',
	'L2', 'L3', 'L4', 'L5', 'L7', 'L8', 'LX', 'L9', 'E0', 'E1', 'E2', 'E3',
	'E4', 'E5', 'E7'),
	'2000': ('M3', 'M4', 'M5', 'M6', 'M8', 'L0', 'L1', 'L6'),
	'2001': ('M1', 'M2a', 'M2b', 'M3', 'M4', 'M5', 'M6a', 'M6b', 'M7a',
	'M7b', 'M8', 'MX', 'L1', 'L2', 'L3', 'L4', 'L5', 'L6', 'L7', 'L8',
	'E1','E2', 'E3', 'E4'),
	'2002': ('M2a', 'M2b', 'M3', 'M4', 'M5', 'M6', 'M7', 'M8',
	'L1', 'L2', 'L3', 'L4', 'L5', 'LX', 'E1'),
	'2003': ('M2a', 'M2b', 'M2c', 'M2d', 'M3a', 'M3b', 'M3c', 'M4a',
	'M4b', 'M5a', 'M5b', 'M6a', 'M6b', 'M7a', 'M7b', 'M8', 'MX', 
	'L1', 'L2a', 'L2b', 'L3a', 'L3b', 'L4', 'L5a', 'L5b', 'L6a', 'L6b',
	'L7', 'L8', 'LX', 'E1a', 'E1b', 'E2a', 'E2b', 'E3', 'E4', 'E5',
	'E7', 'E8', 'EX'),
	'2004': ('M1a', 'M1b', 'M2a', 'M2b', 'M3a', 'M3b', 'M4a', 'M4b', 'M5',
	'M6a', 'M6b', 'M7', 'M8a', 'M8b', 'L1', 'L2', 'L3a', 'L3b', 'L4', 'L5',
	'L6', 'L7', 'L8', 'LX', 'E1', 'E2'),
	'2005': ('M1a', 'M1b', 'M2', 'M3a', 'M3b', 'M4', 'M5', 'M6',
	'M7a', 'M7b', 'M8', 'MX', 'L1', 'L3', 'L4', 'L6', 'L8', 'LX'),
	'2006': ('M2a', 'M2b', 'M3', 'M4', 'M5', 'L1'),
	'2007': ('M1', 'M2', 'M3', 'M4', 'M5', 'M6', 'L2', 'L4', 'L8')}

years = ['1997', '1998', '1999', '2000', '2001', '2002', '2003', '2004',
	'2005', '2006','2007']


games = {}
full_glist = []
names = {}
read_names()

for y in years:
    games[y] = map(DipGame, files[y], [y]*len(files[y]))
    for g in games[y]:
	f = open('s' + y + '/' + g.name + '.html', 'w')
	g.score()
	g.print_summary(f)
	f.close()
	f = open('s' + y + '/s' + y +  '.html','w')
	make_league_table(games[y], y, f)
	f.close()
    full_glist = full_glist + games[y]

f = open('hist/hist.html', 'w')
make_hist_league_table(full_glist, f)
f.close()

f = open('hist/front.html', 'w')
make_hist_front(full_glist, f)
f.close()

f = open('hist/names_u', 'w')

for n in names.keys():
    if names[n] == 'u':
	f.write(n + '\n')
f.close()
